/* fuga.h
 *
 * Vector-International Fuga Camera
 * http://www.vector-international.be/C-Cam/PCI_card/PCIDOC.html
 *
 */
#ifndef FUGA_H
#define FUGA_H

#define FUGA_VENDOR  0x10E8
#define FUGA_DEVICE  0x5920

#define FUGA_MAX_SIZE 512   /* maximum image size */

#define FUGA_YSTART  0x00   /* Y upper-left pixel : min 0 max 511 */
#define FUGA_YINC    0x02   /* Y step (bits 7-0) */
                            /* + ADC delay (bits 15-12) suggested 6 */
#define FUGA_YEND    0x04   /* Y lower-right pixel */
#define FUGA_YWAIT   0x06   /* Vert delay after changing line */
#define FUGA_XSTART  0x08   /* X upper-left pixel */
#define FUGA_XINC    0x0a   /* X step */
#define FUGA_XEND    0x0c   /* X lower-right pixel */
#define FUGA_XWAIT   0x0e   /* nornal H delay between two pixels (bits 7-0) */
                            /* extra H delay (bits 15-8) */
#define FUGA_DATA    0x10   /* pixel data */
#define FUGA_DATA2   0x12
#define FUGA_CTRL    0x14   /* control and status registers */
#define FUGA_STATUS  FUGA_CTRL
#define FUGA_GAIN    0x16   /* gain to subtract from the analogue output signal
                               of the sensor */
#define FUGA_DPA     0x18   /* direct pixel access : when reading bit-15 is
                               the busy flag (1=busy) bits 7-0 are the value */
#define FUGA_DPA2    0x1a
#define FUGA_DPA_BUSY      0x8000  /* DPA busy flag */

/*
 * pixel indices masks
 */
#define FUGA_START_MASK    0x01ff
#define FUGA_INC_MASK      0x00ff
#define FUGA_END_MASK      0x01ff
#define FUGA_ADDR_MASK     0x01ff
/*
 * delays are expressed in 1/4 (system) clock cycle with 33MHz max
 */
#define FUGA_ADCDELAY_MASK  0x000f
#define FUGA_VDELAY_MASK    0x03ff
#define FUGA_HDELAY_MASK    0x00ff
#define FUGA_GAIN_MASK      0x00ff

/* When a value is sent to FUGA_CTRL, bit 15 determines if it is a set (=1)
 * or a clear (=0) Therefore use the following
 * to SET          X | FUGA_CTRL_CTRL_ON    or   X | FUGA_SET
 * to CLR          X & FUGA_CLR             or   X
 */
#define FUGA_SET   0x8000 
#define FUGA_CLR   0xefff
#define FUGA_CTRL_ON 0x8000

#define FUGA_CAMERA_MASK  0x0003 /* select camera on a multi-camera PCI */
#define FUGA_CTRL_CAMERA_MASK  0x0003 /* select camera on a multi-camera PCI */

#define FUGA_FLAGS_MASK        0x1f
#define FUGA_CAMERA_FLAGS      0x03
#define FUGA_CALIBRATION_FLAG  0x04
#define FUGA_LOOPBACK_FLAG     0x08
#define FUGA_CORRECTION_FLAG   0x10

#define FUGA_CTRL_CAMERA_CYCLE 0x0080 /* start/abort input from camera */
#define FUGA_CTRL_CAMERA_ADDRESS 0x0100 /* loopback X-address */
#define FUGA_CTRL_CAMERA_CORRECTION 0x0200 /* perform calibration correction */

#define FUGA_CTRL_FLASH_WRITE  0x0020 /* enable write into card flash-mem */ 
#define FUGA_CTRL_FLASH_READ   0x0000
#define FUGA_CTRL_FLASH_CYCLE  0x0040 /* status and control flash-mem access */

#define FUGA_STATUS_CAMERA_XDRDY 0x1000 /* data in buffer full | EOL reached */

#define FUGA_CTRL_COLOR   0x1000
#define FUGA_CTRL_TELE    0x0800
#define FUGA_CTRL_SNEL    0x0400
#define FUGA_CTRL_2X2_GR  0x1000

#define FUGA_CTRL_CONFIG  0x0004

#define FUGA_FLASH_ADDRESS_L  FUGA_XSTART
#define FUGA_FLASH_ADDRESS_U  FUGA_YSTART
#define FUGA_FLASH_DATA       FUGA_DATA

#define FUGA_FLASH_CMD_WRITE    0x40  /* flash device operation codes */
#define FUGA_FLASH_CMD_ERASE    0x20
#define FUGA_FLASH_CMD_CONFIRM  0xd0
#define FUGA_FLASH_CMD_CLEAR    0x50
#define FUGA_FLASH_CMD_NORMAL   0xff

#define FUGA_FLASH_STATUS_READY 0x80
#define FUGA_FLASH_STATUS_WSMS  0x80
#define FUGA_FLASH_STATUS_ESS   0x40
#define FUGA_FLASH_STATUS_ES    0x20
#define FUGA_FLASH_STATUS_BWS   0x10
#define FUGA_FLASH_STATUS_VPP   0x08

#define FUGA_FLASH_WRITE_TIMEOUT  0x60     // 0x50
#define FUGA_FLASH_ERASE_TIMEOUT  0xf0000  // 0xe0000

#define HIGH_BYTE(ax) (ax >> 8)
#define LOW_BYTE(ax)  (ax & 0xff)

#define FUGA_COLOR  0x04  /* as case 'm' in pci.c */
#define FUGA_TELE   0x02
#define FUGA_SNEL   0x01

#endif /* FUGA_H */
