/* fuga_def.h
 *
 * author marco corvi ( marco_corvi@geocities.yahoo.com )
 * date
 *
 */
#ifndef FUGA_DEF_H
#define FUGA_DEF_H

#define AMCC_SIZE 64
#define FUGA_BUFFER_SIZE 512

typedef struct fuga_amcc {
  unsigned short vendor_id;
  unsigned short device_id;
  unsigned char  nu1;
  unsigned char  bus_master_config;
  unsigned char  nu2[2];
  unsigned char  rev;
  unsigned char  class_code[3];
  unsigned char  nu3;
  unsigned char  latency;
  unsigned char  header_type;
  unsigned char  bist;
  unsigned int address[6];         /* six addresses */
  unsigned char   nu4[8];
  unsigned int rom_addr;
  unsigned char   nu5[8];            /* reserved */
  unsigned char  irq;                /* irq line */
  unsigned char  pin;                /* irq pin  (y-n) */
  unsigned char  grant;
  unsigned char  lat;
  /* u8 user_spec[9]; */
} amcc_type;

struct fuga_ccam {
  unsigned char flags;
  /* unsigned char camera;      // bits 1-0: 0,1,2,3 */
  /* unsigned char correct;     // bit 2: offset correction 1=yes 0=no */
  /* unsigned char calibration; // bit 3: boolean 1=calibrate, 0=not */
  /* unsigned char loopback;    // bit 4: X address loopback */
  unsigned int ystart;
  unsigned int yend;
  unsigned int yinc;
  unsigned int xstart;
  unsigned int xend;
  unsigned int xinc;
  unsigned int vdelay;    /* vertical delay */
  unsigned int hdelay;    /* h delay */
  unsigned int xhdelay;   /* h delay after 32 columns */
  unsigned int adcdelay;
  /* unsigned int data[4];     // four data int's */
  unsigned char gain;
  unsigned int xaddr;
  unsigned int yaddr;
} ccam_type;

struct fuga_device {
  unsigned int   use_count;      /* usage count */ 
  unsigned int   bus_nr;         /* bus number */
  unsigned int   fun_nr;         /* function number */
  struct fuga_amcc amcc;
  struct fuga_ccam ccam;
  unsigned long  timeouts;
};


/*
 * function prototypes
 */
#ifdef __KERNEL__
int fuga_open( struct inode *inode, struct file * filp); 
int fuga_release(struct inode *inode, struct file *filp);
ssize_t fuga_read(struct file *filp, char * buf, size_t cnt, loff_t * off); 
int fuga_ioctl(struct inode*inode, struct file *filp, 
    unsigned int cmd, unsigned long arg);

int fuga_hw_init( struct fuga_device * dev );
int fuga_ccam_reset( struct fuga_device * dev );
int fuga_ccam_validate( struct fuga_device * dev );

int fuga_flash_read( struct fuga_device * dev, u8 * buf);
int fuga_flash_write( struct fuga_device * dev, u8 * buf);
int fuga_flash_clear( struct fuga_device * dev);
int fuga_flash_test( struct fuga_device * dev );

int fuga_flash_status( struct fuga_device * dev, unsigned long to );
int fuga_flash_erase( struct fuga_device * dev, u32 addr);
int fuga_flash_byte_write( struct fuga_device * dev, u16 x, u16 y, char data );
int fuga_flash_byte_read( struct fuga_device * dev, u16 x, u16 y );
int fuga_flash_reset( struct fuga_device * dev );

int fuga_reset     ( struct fuga_device * dev );
int fuga_set_flags ( struct fuga_device * dev );
int fuga_set_woi   ( struct fuga_device * dev );

int fuga_flex_ttf( struct fuga_device * dev, u8 * buf);
int fuga_flex_config( struct fuga_device * dev);
int fuga_novram_write( struct fuga_device * dev);

#endif

#endif
