/* fuga_test.c
 *
 * test program for fuga device driver
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <sys/ioctl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/time.h>
#include <fcntl.h>
#include <unistd.h>
#include <errno.h>

#include "fuga_def.h"
#include "fuga_ioctl.h"
#include "fuga.h"

struct fuga_ccam ccam;
struct fuga_amcc amcc;

unsigned char buf[512*512];
char calbuf[512*512];
unsigned char img[512*512];
int xcount=512, ycount=512;
int cnt = 512*512;

struct timeval tvs, tve;
long dtime;


/* GRAPHICS */
int drawToGui(unsigned char * buf, int dx, int dy);
int initGraphics();
int drawMenu();
int drawMsg( char *, int );
int mouseClick();
int getClickAmount();
int printGui( char * msg );

int do_read( int );

char msg0[120];
char msg1[120];

void reset_count() {
  xcount = (ccam.xend - ccam.xstart + ccam.xinc)/ ccam.xinc;
  ycount = (ccam.yend - ccam.ystart + ccam.yinc)/ ccam.yinc;
  cnt = xcount*ycount;
}


int set_ttf( int fd ) 
{
   int isnr, nr, k, i;
   char * ttfname = "ccamemc.ttf";
   FILE * ttfp;
   unsigned char ttfbuf[FUGA_TTF_SIZE];
   int n=0;

   if ( (ttfp = fopen( ttfname, "r") ) == NULL) {
     sprintf(msg1, "unable to open TTF file %s          ", ttfname );
     drawMsg( msg1, 1);
     return (-1);
   } 
   isnr = nr = n = 0;
   i = 0;
   while ( ((k = fgetc( ttfp )) != EOF) && (n < FUGA_TTF_SIZE) ) {
     i ++;
     if (isdigit(k)) {
       if (isnr == 0) {
         isnr = 1;
         nr = k - '0';
       } else {
         nr = nr*10 + k - '0';
       }
     } else {
       if (isnr != 0) {
         ttfbuf[n++] = (unsigned char) nr;
         isnr = nr = 0;
       } else { 
         /* discard */
       }
     }
   } 
   fclose( ttfp );

   if ( n != FUGA_TTF_SIZE ) {   /* k may be != EOF */
     sprintf(msg1, "Incomplete TTF read: %d bytes C=%d         ", n, k);
     drawMsg( msg1, 1);
     return (-2);
   }
   
   gettimeofday( &tve, NULL );
   if ( (n = ioctl(fd, FUGA_IOCS_TTF, ttfbuf)) < 0) {
     printf(" ioctl TTF returns %d \n", n);
   }
   gettimeofday( &tvs, NULL );
   dtime = 1000000L*(tvs.tv_sec - tve.tv_sec) 
         + tvs.tv_usec - tve.tv_usec;
   sprintf(msg0, "Written ttf in %ld usec                   ", dtime);
   drawMsg( msg0, 0);

   return n;
}

int do_store(int fd ) {
  int i,k;
  FILE * fnip;
  char header[6] = { 1, 2, 0, 2, 0, 1 };
 
  for (k=0; k<10; k++) {
    do_read( fd );
    for (i=0; i<512*512; i++) 
      calbuf[i] = (char)(0.9*calbuf[i] + 0.1*(128-(int)buf[i]) );
  }

  fnip = fopen("fugatest.nip", "w");
  fwrite(header, 1, 6, fnip);
  fwrite(calbuf, 1, 512*512, fnip);
  fclose( fnip );
  return 0;
}

int do_zoom( int fd, int zoom)
{ int retval = 0;
  if (zoom == +1 && ccam.xinc>1) {
    ccam.xinc >>= 1;
    ccam.yinc >>= 1;
    retval = ioctl( fd, FUGA_IOCS_CCAM, &ccam);
  } else if (zoom == -1 && ccam.xinc<64) {
    ccam.xinc <<= 1;
    ccam.yinc <<= 1;
    retval = ioctl( fd, FUGA_IOCS_CCAM, &ccam);
  }
  reset_count();
  return retval;
}

int do_gain( int fd , int gain) 
{ int retval = 0;
  if (gain > 0 && ccam.gain<(250-gain*5)) {
    ccam.gain += gain*5;
    retval = ioctl( fd, FUGA_IOCS_CCAM, &ccam);
  } else if (gain < 0 && ccam.gain>(10-gain*5)) {
    ccam.gain += gain*5;
    retval = ioctl( fd, FUGA_IOCS_CCAM, &ccam);
  }
  return retval;
}

int do_vdelay( int fd , int delay) 
{ int val;
  val = ccam.vdelay + delay;
  if (val > 1023) val = 1023;
  if (val < 1)    val = 1;
  ccam.vdelay = val;
  return ioctl( fd, FUGA_IOCS_CCAM, &ccam);
}

int do_hdelay( int fd , int delay) 
{ int val;
  val = ccam.hdelay + delay;
  if (val > 255) val = 255;
  if (val < 0)   val = 0;
  ccam.hdelay = val;
  return ioctl( fd, FUGA_IOCS_CCAM, &ccam);
}

int do_xhdelay( int fd , int delay) 
{ int val;
  val = ccam.xhdelay + delay;
  if (val > 255) val = 255;
  if (val < 0)   val = 0;
  ccam.xhdelay = val;
  return ioctl( fd, FUGA_IOCS_CCAM, &ccam);
}

int do_adcdelay( int fd , int delay) 
{ 
  if ( delay > 0 ) {
    ccam.adcdelay += 1;
  } else if (delay < 0) {
    ccam.adcdelay -= 1;
  }
  if (ccam.adcdelay > 15) ccam.adcdelay = 15;
  if (ccam.adcdelay < 3) ccam.adcdelay = 3;
  return ioctl( fd, FUGA_IOCS_CCAM, &ccam);
}

int get_pixel( int fd ) 
{
  unsigned int x, y;
  int val;

  printf("Enter Y-X: ");
  scanf("%d%d", &y, &x);
  x &= 0x01ff;
  y &= 0x01ff;

  val = (y<<9)+x;
  sprintf(msg0, " ioctl %d                                ", 
    ioctl(fd, FUGA_IOCG_PIXEL, &val) );
  drawMsg( msg0, 0 );
  return val;
}

int toggle_flags(int toggle) {
  ccam.flags ^= toggle;
  return 0;
}

int do_flags( int fd ) {
  ioctl(fd, FUGA_IOCS_CCAM, &ccam);
  return 0;
}

int do_dump(int fd) {  
   int n1=0, n2=0;
   char infomsg[256];
   
   if ( (n1 = ioctl(fd, FUGA_IOCG_CCAM, &ccam)) == 0 && 
        (n2 = ioctl(fd, FUGA_IOCG_AMCC, &amcc)) == 0 ) {
     reset_count(); /* not necessary ... */
     sprintf(infomsg, 
       "Vendor %04x Device %04x Addr. %04x %04x %04x %04x\n"
       "Camera %1d Flags %s %s %s Gain %d           \n"
       "WoI Y: %d %d %d  X: %d %d %d  Delays %d %d %d %d\n",
       amcc.vendor_id, amcc.device_id, amcc.address[0], amcc.address[1],
       amcc.address[2], amcc.address[3],
       ccam.flags & 3, 
       (ccam.flags & FUGA_CORRECTION_FLAG) ? "correct" : " ",
       (ccam.flags & FUGA_CALIBRATION_FLAG) ? "calibr." : " ",
       (ccam.flags & FUGA_LOOPBACK_FLAG) ? "loopback" : " ",
       ccam.gain,
       ccam.ystart, ccam.yinc, ccam.yend,
       ccam.xstart, ccam.xinc, ccam.xend,
       ccam.vdelay, ccam.hdelay, ccam.xhdelay, ccam.adcdelay );
   } else {
     sprintf(infomsg, " ioctl IOCG error %d %d \n \n \n", n1, n2);
   }
   printGui( infomsg );
   return n1+n2;
}
 
int do_reset(int fd) {
   int retval;

   sprintf(msg0, 
     "Hw reset %d  Sw reset %d                       ", 
     ioctl(fd, FUGA_IOC_HW_RESET), ioctl(fd, FUGA_IOC_RESET)
   );
   drawMsg( msg0, 0 );
   
   retval = ioctl( fd, FUGA_IOCG_CCAM, &ccam);
   reset_count();
   return retval;
}


int do_read( int fd ) {
  int n;
  gettimeofday( &tve, NULL );
  n = read(fd, buf, cnt);
  gettimeofday( &tvs, NULL );
  dtime = 1000000L*(tvs.tv_sec - tve.tv_sec) 
        + tvs.tv_usec - tve.tv_usec;
  sprintf(msg0, "Read %d bytes in %ld usec                ", n, dtime);
  drawMsg( msg0, 0 );
  sprintf( msg1, "                                                ");
  drawMsg( msg1, 1 );
  return n;
}

int do_motion( int fd ) {
  int c;

  sprintf( msg0, "Reading a sequence of 20 frames,                ");
  drawMsg( msg0, 0 );
  sprintf( msg1, "It takes about 5 secs                           ");
  drawMsg( msg1, 1 );

  for (c=0; c<20; c++) {
    if ( read(fd, buf, cnt) > 0) {
      drawToGui( buf, xcount, ycount);
    }
  }
  return 0;
}


int do_write(int fd) {
  int n;
  FILE * fp;
  
  fp = fopen("fugacal.nip", "r");
  fread(buf, 1, 6, fp);
  if ( ( n = fread(calbuf, 1, 512*512, fp) ) < 512*512) {
    sprintf(msg1, "NIP file error: read %d bytes          ", n);
    drawMsg( msg1, 1);
    return -EIO;
  }
  for (n=0; n<512*512; n++) 
    calbuf[n] = (char)(128 - (int)calbuf[n]);

  sprintf( msg0, "Writing calibration image,                     ");
  drawMsg( msg0, 0 );
  sprintf( msg1, "It takes a while. Please wait, ...              ");
  drawMsg( msg1, 1 );

  gettimeofday( &tve, NULL );
  n = ioctl(fd, FUGA_IOCS_CALIBRATION, calbuf);
  gettimeofday( &tvs, NULL );
  dtime = 1000000L*(tvs.tv_sec - tve.tv_sec) 
        + tvs.tv_usec - tve.tv_usec;
  sprintf(msg0, "Written (%04x) in %ld usec                    ", 
    (n<0)? n^(-0xffff) : n, dtime);
  drawMsg( msg0, 0 );
  return n;
}

int do_cal_reset() {
  int i;
  for (i=0; i<512*512; i++) 
    calbuf[i] = 0;
  return 0;
}

int show_again( int fd ) {
  do_dump( fd );
  do_read( fd );
  drawToGui( buf, xcount, ycount);
  return 0;
}

int
main( int argc, char ** argv ) 
{
  int do_quit=0;
  // static unsigned char ch = ' ';
  // int n=0;
  char * devname = "/dev/fuga";
  int fd;

  if ( (fd = open( devname, O_RDONLY)) < 0) {
    printf("open error %d\n", fd);
    exit(0);
  }
  ioctl(fd, FUGA_IOCG_CCAM, &ccam);
  reset_count();
  do_cal_reset();

  initGraphics();
  drawMenu();

  while (do_quit == 0) {
    switch( mouseClick() ) {
      case 0: // BOOT
              ioctl( fd, FUGA_IOC_CONFIG);
              break;
      case 1: // HW+SW RESET and CAL_RESET
              do_reset(fd);
              ioctl(fd, FUGA_IOCG_CCAM, &ccam);
              reset_count();
              do_cal_reset();
              do_dump(fd);
              break;
      case 2: // TTF
              set_ttf( fd );
              break;
      case 3: // STORE and SAVE
              // reads ten times the image and asve it to fugatest.nip
              do_store( fd );
              break;
      case 4: // WRITE
              // write calibration image (from file fugacal.nip)
              sprintf(msg1, "write calibration %d                  ",
                do_write(fd ) );
              drawMsg( msg1, 1);
              break;
      case 5: // CLEAR
              // clear calibration image
              sprintf(msg0, 
                "clear calibration %d                                ", 
                ioctl(fd, FUGA_IOC_CLEAR) );
              drawMsg( msg0, 0);
              break;
      case 6: // TEST
              // test write/read calibration
              sprintf(msg0, "test calibration %d                     ", 
                ioctl(fd, FUGA_IOC_TEST) );
              drawMsg( msg0, 0);
              break;

      case 16: // READ 
              // read and display one frame
              do_read( fd );
              drawToGui( buf, xcount, ycount);
              break;
      case 17: // MOTION
              // read and display 20 frames
              do_motion( fd );
              break;
      case 18: // CORRECT FLAG
              toggle_flags( FUGA_CORRECTION_FLAG);
              do_flags(fd);
              show_again( fd );
              break;
      case 19: // LOOPBACK FLAG
              toggle_flags( FUGA_LOOPBACK_FLAG);
              do_flags(fd);
              show_again( fd );
              break;
      case 20: // CALIBRATION FLAG
              toggle_flags( FUGA_CALIBRATION_FLAG);
              do_flags(fd);
              show_again( fd );
              break;
      case 21: // GET PIXEL
              // printf("get_pixel %d \n", get_pixel(fd) );
              printf("Get Pixel not implemented %c%c%c\n", 7,7,7);
              break;
      case 22: // QUIT 
              do_quit = 1;
              break;

      case 32: // ZOOM + 
      case 33: // ZOOM - 
              do_zoom( fd, getClickAmount());
              show_again( fd );
              break;
      case 34: // GAIN +
      case 35: // GAIN -
              do_gain(fd, getClickAmount());
              show_again( fd );
              break;
      case 36: // V-DELAY +
      case 37: // V-DELAY -
              do_vdelay( fd, getClickAmount());
              do_dump( fd );
              break;
      case 38: // H-DELAY +
      case 39: // H-DELAY -
              do_hdelay( fd, getClickAmount());
              do_dump( fd );
              break;
      case 40: // XH-DELAY +
      case 41: // XH-DELAY -
              do_xhdelay( fd, getClickAmount());
              do_dump( fd );
              break;
      case 42: // ADC-DELAY +
      case 43: // ADC-DELAY -
              do_adcdelay( fd, getClickAmount());
              do_dump( fd );
              break;
    }
  }
  close(fd);
  exit(0);
}
