/* 

   FUGA Graphical User Interface (GUI)

   Author: marco corvi (marco_corvi@geocities.yahoo.com)
   Date  : 

   BUGS:
     not the best gui at all, just a quick and dirty graphical interface
     because one needs to see some images

   Read the explanations before the menu's (below)   
*/

/* ------------------------------------------------------------ */

#include <stdio.h>        // for sprintf, sscanf, fclose 
#include <stdlib.h>       // for malloc
#include <string.h>       // for strcpy

/* ======= LAYOUT SPECIFICATIONS =================== */
#define DX 512     // image size
#define DY 512

#define WCTR   400    // control window size
#define HCTR   220

#define YMENU1  20
#define YMENU2  40
#define YMENU3  60
int YMENU[3] = {YMENU1, YMENU2, YMENU3};
int YINFO[3] = {100, 120, 140};
#define WITEM  55

#define XMSG   0
#define YMSG   180

#define WIMG   512    // image/histogram window size
#define HIMG   622    // 512 + 100 + 10
#define YHIST  110    // baseline for the histogram
#define YIMAGE 110    // topline for image == YHIST

/* ============== X WINDOW STUFF ==================== */
#include <X11/Xlib.h>
#include <X11/Xos.h>
#include <X11/Xutil.h>
#include <X11/X.h>
#include <X11/cursorfont.h>
#include <X11/Xatom.h>

#ifndef TRUE
#define TRUE 1
#endif
#ifndef FALSE
#define FALSE 0
#endif

Display * theDisplay;       // the graphic display
GC        theGC;            // the graphic context
XGCValues xgcv;
int       theScreen;        // the display screen
int       theDepth;         // the screen depth (8 bit planes ???)
Window    rootW;            // the root window
Window    theWindow;        // the window for the image/histogram
Window    theWCTRL;         // the window to play the game 
Visual  * theVisual;
Colormap  theCmap;          // the default colormap
Colormap  privCmap;         // private colormap
Font      theFont;          // display font
XSetWindowAttributes xswa;
unsigned long flag;

int ncolor;  /* number of colors */
XColor theColor[256]; 

XEvent    event1, event2;   // a couple of X-events


int offset=0;

/*
 * config 
 * reset   hw/sw reset
 * init    load ttf file
 * store   read several times the image and writes to file (nip)
 * write   write calibration fugacal.nip
 * clear   clear calibration
 * test    test calibration i/o
 * 
 * Missing read-calibration
 */
char * menu1[] = {
  "config", "reset", "init", "store", "write", "clear", "test",  NULL 
};

/*
 * read    read camera image
 * motion  read 20 frames in a row
 * correct  toggle correction flag
 * loopback toggle loopback flag
 * calibr.  toggle calibration flag (what is it for ?)
 * pixel    read a single pixel (interface not good yet)
 * quit     exit
 */
char * menu2[] = {
  "read", "motion", "correct", "loopback", "calibr.", "pixel", "quit", NULL
};

/*
 * zoom    increase/decrease zoom 
 * gain
 * V       vert (Y) delay
 * H       horiz. delay
 * XH      extra horiz. delay
 * Adc     ADC delay
 *
 * Note: for the underlined word click-n-drag the mouse across the
 *       word in the direction of change (prop to the change)
 *       For the others click on '+' or '-' 
 */
char * menu3[] = {
  "-zoom+", "-gain+", "- V +", "- H +", "- XH +", "-Adc+", NULL
};

int underline[] = { 0, 1, 1, 1, 1, 0 };

int menu1Number = 7;
int menu2Number = 7;
int menu3Number = 6;

int hist[256];  /* histogram */
int histmax;
  

/* ------------------------------------------------------------------ */ 
int drawHist() {
  int i;
  XSetForeground(theDisplay, theGC, theColor[255].pixel);
  for (i=0; i<256; i++) {
    XDrawPoint(theDisplay, theWindow, theGC,
      i, YHIST-(int)((100.0*hist[i])/histmax) );
  }
  return 0;
}

int drawTitle() {
  char title[40];
  sprintf(title," FUGA Image");
  XStoreName(theDisplay, theWindow, title);
  sprintf(title," FUGA Control");
  XStoreName(theDisplay, theWCTRL, title);
  XFlush(theDisplay);
  return 0;
}

/* ------------------------------------------------------------------ */ 
int initGraphics1() {
  int theClass;
  theDisplay = XOpenDisplay(XDisplayName(NULL));
  if (theDisplay == NULL) {
    printf("You need to run the program under X-window.\n\n");
    exit(1);
  }
  theScreen  = XDefaultScreen(theDisplay);
  rootW      =  RootWindow(theDisplay, theScreen);
  theGC      = XDefaultGC(theDisplay, theScreen);
  theCmap    = XDefaultColormap(theDisplay, theScreen);
  theVisual  =  DefaultVisual(theDisplay, theScreen);
  theDepth   = XDefaultDepth(theDisplay, theScreen);

  theFont    = XLoadFont(theDisplay, "lucidasans-10");
  XSetFont(theDisplay, theGC, theFont);

  theClass = theVisual->class;

  if (theClass == PseudoColor) printf(" Visual PseudoColor \n");
  else if (theClass == GrayScale)   printf(" Visual GrayScale \n");
  else if (theClass == DirectColor) printf(" Visual DirectColor  \n");
  else if (theClass == StaticGray)  printf(" Visual StaticGray \n");
  else if (theClass == StaticColor)  printf(" Visual StaticColor \n");
  else printf(" Visual unknown %d \n", theClass);

  return(0);
}

int look_up[256];
int lk_up=0;

int initGraphics2() {
  int i, j;
  ncolor = 0;

  {
    privCmap = XCreateColormap( theDisplay, rootW, theVisual, AllocNone);

    for (i=0; i<256; i++) {
      theColor[i].red   = (65536/256)*i;
      theColor[i].green = (65536/256)*i;
      theColor[i].blue  = (65536/256)*i;
      theColor[i].flags = DoRed | DoGreen | DoBlue;

      ncolor += XAllocColor(theDisplay, privCmap, &theColor[i]);

      j = 0;
      while ( (j<i) && (
            (theColor[i].red   != theColor[j].red) ||
            (theColor[i].green != theColor[j].green) ||
            (theColor[i].blue  != theColor[j].blue) ) )
        j++;
      if (j==i)
      {
        look_up[i]=lk_up;
        lk_up++;
      } else {
        look_up[i]=look_up[j];
      }
    }
    printf("Allocated %d colors \n", ncolor);

  }


  return(0);
}


int initGraphics3() {
  XSynchronize(theDisplay, 1);

  xswa.backing_store = Always;
  xswa.event_mask    = ExposureMask | ButtonPressMask | ButtonReleaseMask ;
  xswa.background_pixel  = theColor[0].pixel;
  xswa.save_under        = TRUE;
  xswa.override_redirect = FALSE;
  xswa.colormap          = theCmap;
  flag = CWBackingPlanes | CWDontPropagate | CWSaveUnder | CWBackingStore |
         CWOverrideRedirect | CWEventMask  | CWCursor    | CWBackPixel |
         CWBorderPixel | CWColormap ;
  theWindow = XCreateWindow(theDisplay, rootW, 0, 0, WIMG, HIMG, 0, theDepth,
         InputOutput, theVisual, flag, &xswa);
  XMapWindow(theDisplay, theWindow);
  theWCTRL = XCreateWindow(theDisplay, rootW, 0, 0, WCTR, HCTR, 0, theDepth,
         InputOutput, theVisual, flag, &xswa);
  XMapWindow(theDisplay, theWCTRL);

  return(0);
}

  

int initGraphics4() {
  XSetWindowColormap( theDisplay, theWindow, privCmap);
  XSetWindowColormap( theDisplay, theWCTRL, privCmap);
  return(0);
}

int drawMenu() {
  int x;

  XSetBackground(theDisplay, theGC, theColor[0].pixel);
  XSetForeground(theDisplay, theGC, theColor[255].pixel);
  for (x=0; menu1[x] != NULL; x++) {
    XDrawImageString(theDisplay, theWCTRL, theGC,
      5+WITEM*x, YMENU1, menu1[x], strlen(menu1[x]) );
  }
  for (x=0; menu2[x] != NULL; x++) {
    XDrawImageString(theDisplay, theWCTRL, theGC,
      5+WITEM*x, YMENU2, menu2[x], strlen(menu2[x]) );
  }
  for (x=0; menu3[x] != NULL; x++) {
    XDrawImageString(theDisplay, theWCTRL, theGC,
      5+WITEM*x, YMENU3, menu3[x], strlen(menu3[x]) );
    if (underline[x]) {
      XDrawLine( theDisplay, theWCTRL, theGC,
        5+WITEM*x, YMENU3+2, WITEM*(x+1), YMENU3+2);
    }
  }
  return(0);
}

int drawMsg( char * msg, int line ) {
  if (line >= 0 && line <= 1) {
    XSetBackground(theDisplay, theGC, theColor[0].pixel);
    XSetForeground(theDisplay, theGC, theColor[255].pixel);
    XDrawImageString(theDisplay, theWCTRL, theGC,
      5+XMSG, YMSG+20*line, msg, strlen(msg) );
  }
  return 0;
}

/* ------------------------------------------------------------------ */

int printGui( char * msg ) 
{
  char * ch, *ch1;
  int i = 0;

  XClearWindow(theDisplay, theWCTRL); 
  XSetBackground(theDisplay, theGC, theColor[0].pixel);
  XSetForeground(theDisplay, theGC, theColor[255].pixel);
  drawMenu();
  ch1 = msg;
  for (ch=msg; ch!=0 && i<3; ch++) {
    if (*ch == '\n') {
      // *ch = 0;
      XDrawImageString(theDisplay, theWCTRL, theGC,
        5, YINFO[i], ch1, (int)(ch-ch1) );
      ch1 = ch+1;
      i ++;
    }
  }
  return i;
}
    

int drawToGui(unsigned char * buf, int dx, int dy)
{
  XImage * xi; 
  int i;
  
  for (i=0; i<256; i++) hist[i]=0;

  for (i=0; i<dx*dy; i++) {
    hist[ buf[i] ] ++;
    buf[i] = (unsigned char)(look_up[buf[i]]);
  }

  histmax = 0;
  for (i=0; i<256; i++) {
    if (histmax < hist[i]) histmax = hist[i];
  }

  XClearWindow(theDisplay, theWindow); 
  xi = XCreateImage(theDisplay, theVisual, theDepth, ZPixmap,
      0, (char *)(buf), dx, dy, 8, 0);
  XPutImage( theDisplay, theWindow, theGC, xi, 0, 0, 0, YIMAGE, dx, dy); 
  drawHist();

  XFlush( theDisplay );
  return(0);
}

int initGraphics() {
  initGraphics1();
  initGraphics2();
  initGraphics3();
  initGraphics4();
  drawTitle();
  return 0;
}
/* ------------------------------------------------------------------ */
int clickAmount = 0;

int getClickAmount() {
  return clickAmount;
}

int getIndex(int x1, int y1, int x2, int y2) {
  int x = x1 / WITEM;
  clickAmount = 0;
  if (y2 > YMENU3 || y1 > YMENU3) return -1;
  if (y2 > YMENU2 ) {
    if ( x == 0           /* Zoom */
      || x == 5 ) {       /* ADC delay */
      clickAmount = (x2 > (x+0.5)*WITEM) ? 1 : -1;
    } else {              /* all others */
      clickAmount = ((x2 - x1)*15)/WITEM;
    }
    if (y1 > YMENU2 && y1 < YMENU3) {
      if (x < menu3Number && abs(x1-x2) < WITEM) {
        if (x1 < x2) {
          return (2*16 + 2*x); 
        } else {
          return (2*16 + 2*x +1);
        }
      }
    }
  } else if (y2 > YMENU1) {
    if (y1 > YMENU1 && y1 < YMENU2) {
      if (x < menu2Number && abs(x2-x1) < WITEM) {
        return (1*16 + x); 
      }
    }
  } else {
    if (y1 > 0 && y1 < YMENU1) {
      if (x < menu1Number && abs(x2-x1) < WITEM) {
        return (0*16 + x); 
      }
    }
  }
  return -1;
}
    

int mouseClick() {
  int i1=-1;
  do {
    XNextEvent(theDisplay, &event1);
    if (event1.type == ButtonPress) {
      XNextEvent(theDisplay, &event2);
      while (event2.type != 5) XNextEvent(theDisplay, &event2);
      i1 = getIndex(
             event1.xbutton.x, event1.xbutton.y,
             event2.xbutton.x, event2.xbutton.y);
    }
  } while (i1<0);
  return (i1);
}


/* ================================================================== */
/*
 * this main is useful only to see the codes of the mouse clicks
 *

int main( int argc, char ** argv) {
  int i;
  unsigned char * img;
  int dx=0, dy0;

  //
  // read the image img and its size (dx, dy)
  //

  initGraphics();
  // drawToGui(img, dx, dy);
  drawMenu();

  while ( (i = mouseClick() ) >= 0) {
    printf("Click %d \n", i);
  }
  exit(0);
}


*/
